@@ -54,13 +54,13 @@ extern "C" {
 #endif
 /* *INDENT-ON* */
 
-#define GNUTLS_VERSION "3.2.15"
+#define GNUTLS_VERSION "3.3.16"
 
 #define GNUTLS_VERSION_MAJOR 3
-#define GNUTLS_VERSION_MINOR 2
-#define GNUTLS_VERSION_PATCH 15
+#define GNUTLS_VERSION_MINOR 3
+#define GNUTLS_VERSION_PATCH 16
 
-#define GNUTLS_VERSION_NUMBER 0x03020f
+#define GNUTLS_VERSION_NUMBER 0x030310
 
 #define GNUTLS_CIPHER_RIJNDAEL_128_CBC GNUTLS_CIPHER_AES_128_CBC
 #define GNUTLS_CIPHER_RIJNDAEL_256_CBC GNUTLS_CIPHER_AES_256_CBC
@@ -86,6 +86,8 @@ extern "C" {
  * @GNUTLS_CIPHER_AES_256_GCM: AES in GCM mode with 256-bit keys.
  * @GNUTLS_CIPHER_CAMELLIA_128_GCM: CAMELLIA in GCM mode with 128-bit keys.
  * @GNUTLS_CIPHER_CAMELLIA_256_GCM: CAMELLIA in GCM mode with 256-bit keys.
+ * @GNUTLS_CIPHER_SALSA20_256: Salsa20 with 256-bit keys.
+ * @GNUTLS_CIPHER_ESTREAM_SALSA20_256: Estream's Salsa20 variant with 256-bit keys.
  * @GNUTLS_CIPHER_IDEA_PGP_CFB: IDEA in CFB mode.
  * @GNUTLS_CIPHER_3DES_PGP_CFB: 3DES in CFB mode.
  * @GNUTLS_CIPHER_CAST5_PGP_CFB: CAST5 in CFB mode.
@@ -169,7 +171,7 @@ typedef enum {
 	GNUTLS_KX_ECDHE_RSA = 12,
 	GNUTLS_KX_ECDHE_ECDSA = 13,
 	GNUTLS_KX_ECDHE_PSK = 14,
-	GNUTLS_KX_RSA_PSK = 15,
+	GNUTLS_KX_RSA_PSK = 15
 } gnutls_kx_algorithm_t;
 
 /**
@@ -183,7 +185,7 @@ typedef enum {
 typedef enum {
 	GNUTLS_PARAMS_RSA_EXPORT = 1,
 	GNUTLS_PARAMS_DH = 2,
-	GNUTLS_PARAMS_ECDH = 3,
+	GNUTLS_PARAMS_ECDH = 3
 } gnutls_params_type_t;
 
 /**
@@ -241,7 +243,7 @@ typedef enum {
 	   gnutls_digest_algorithm_t. */
 	GNUTLS_MAC_AEAD = 200,	/* indicates that MAC is on the cipher */
 	GNUTLS_MAC_UMAC_96 = 201,
-	GNUTLS_MAC_UMAC_128 = 202,
+	GNUTLS_MAC_UMAC_128 = 202
 } gnutls_mac_algorithm_t;
 
 /**
@@ -292,7 +294,7 @@ typedef enum {
 	GNUTLS_COMP_UNKNOWN = 0,
 	GNUTLS_COMP_NULL = 1,
 	GNUTLS_COMP_DEFLATE = 2,
-	GNUTLS_COMP_ZLIB = GNUTLS_COMP_DEFLATE,
+	GNUTLS_COMP_ZLIB = GNUTLS_COMP_DEFLATE
 } gnutls_compression_method_t;
 
   /*
@@ -359,6 +361,8 @@ typedef enum {
  *   recognized.
  * @GNUTLS_A_UNKNOWN_PSK_IDENTITY: The SRP/PSK username is missing
  *   or not known.
+ * @GNUTLS_A_NO_APPLICATION_PROTOCOL: The ALPN protocol requested is
+ *   not supported by the peer.
  *
  * Enumeration of different TLS alerts.
  */
@@ -391,7 +395,7 @@ typedef enum {
 	GNUTLS_A_CERTIFICATE_UNOBTAINABLE = 111,
 	GNUTLS_A_UNRECOGNIZED_NAME = 112,
 	GNUTLS_A_UNKNOWN_PSK_IDENTITY = 115,
-	GNUTLS_A_NO_APPLICATION_PROTOCOL = 120,
+	GNUTLS_A_NO_APPLICATION_PROTOCOL = 120
 } gnutls_alert_description_t;
 
 /**
@@ -431,7 +435,7 @@ typedef enum {
 	GNUTLS_HANDSHAKE_CERTIFICATE_STATUS = 22,
 	GNUTLS_HANDSHAKE_SUPPLEMENTAL = 23,
 	GNUTLS_HANDSHAKE_CHANGE_CIPHER_SPEC = 254,
-	GNUTLS_HANDSHAKE_CLIENT_HELLO_V2 = 1024,
+	GNUTLS_HANDSHAKE_CLIENT_HELLO_V2 = 1024
 } gnutls_handshake_description_t;
 
 #define GNUTLS_HANDSHAKE_ANY ((unsigned int)-1)
@@ -482,7 +486,7 @@ typedef enum {
 	GNUTLS_CERT_UNEXPECTED_OWNER = 1 << 14,
 	GNUTLS_CERT_REVOCATION_DATA_ISSUED_IN_FUTURE = 1 << 15,
 	GNUTLS_CERT_SIGNER_CONSTRAINTS_FAILURE = 1 << 16,
-	GNUTLS_CERT_MISMATCH = 1 << 17,
+	GNUTLS_CERT_MISMATCH = 1 << 17
 } gnutls_certificate_status_t;
 
 /**
@@ -596,7 +600,7 @@ typedef enum gnutls_certificate_print_formats {
 	GNUTLS_CRT_PRINT_ONELINE = 1,
 	GNUTLS_CRT_PRINT_UNSIGNED_FULL = 2,
 	GNUTLS_CRT_PRINT_COMPACT = 3,
-	GNUTLS_CRT_PRINT_FULL_NUMBERS = 4,
+	GNUTLS_CRT_PRINT_FULL_NUMBERS = 4
 } gnutls_certificate_print_formats_t;
 
 #define GNUTLS_PK_ECC GNUTLS_PK_EC
@@ -615,7 +619,7 @@ typedef enum {
 	GNUTLS_PK_RSA = 1,
 	GNUTLS_PK_DSA = 2,
 	GNUTLS_PK_DH = 3,
-	GNUTLS_PK_EC = 4,
+	GNUTLS_PK_EC = 4
 } gnutls_pk_algorithm_t;
 
 const char *gnutls_pk_algorithm_get_name(gnutls_pk_algorithm_t algorithm);
@@ -628,6 +632,8 @@ const char *gnutls_pk_algorithm_get_name(gnutls_pk_algorithm_t algorithm);
  * @GNUTLS_SIGN_DSA_SHA1: Digital signature algorithm DSA with SHA-1
  * @GNUTLS_SIGN_DSA_SHA224: Digital signature algorithm DSA with SHA-224
  * @GNUTLS_SIGN_DSA_SHA256: Digital signature algorithm DSA with SHA-256
+ * @GNUTLS_SIGN_DSA_SHA384: Digital signature algorithm DSA with SHA-384
+ * @GNUTLS_SIGN_DSA_SHA512: Digital signature algorithm DSA with SHA-512
  * @GNUTLS_SIGN_DSA_SHA: Same as %GNUTLS_SIGN_DSA_SHA1.
  * @GNUTLS_SIGN_RSA_MD5: Digital signature algorithm RSA with MD5.
  * @GNUTLS_SIGN_RSA_MD2: Digital signature algorithm RSA with MD2.
@@ -664,6 +670,8 @@ typedef enum {
 	GNUTLS_SIGN_ECDSA_SHA256 = 14,
 	GNUTLS_SIGN_ECDSA_SHA384 = 15,
 	GNUTLS_SIGN_ECDSA_SHA512 = 16,
+	GNUTLS_SIGN_DSA_SHA384 = 17,
+	GNUTLS_SIGN_DSA_SHA512 = 18
 } gnutls_sign_algorithm_t;
 
 /**
@@ -683,7 +691,7 @@ typedef enum {
 	GNUTLS_ECC_CURVE_SECP256R1,
 	GNUTLS_ECC_CURVE_SECP384R1,
 	GNUTLS_ECC_CURVE_SECP521R1,
-	GNUTLS_ECC_CURVE_SECP192R1,
+	GNUTLS_ECC_CURVE_SECP192R1
 } gnutls_ecc_curve_t;
 
 /* macros to allow specifying a specific curve in gnutls_privkey_generate()
@@ -701,7 +709,7 @@ typedef enum {
  * @GNUTLS_SEC_PARAM_WEAK: 72 bits of security
  * @GNUTLS_SEC_PARAM_LOW: 80 bits of security
  * @GNUTLS_SEC_PARAM_LEGACY: 96 bits of security
- * @GNUTLS_SEC_PARAM_NORMAL: 112 bits of security
+ * @GNUTLS_SEC_PARAM_MEDIUM: 112 bits of security (used to be %GNUTLS_SEC_PARAM_NORMAL)
  * @GNUTLS_SEC_PARAM_HIGH: 128 bits of security
  * @GNUTLS_SEC_PARAM_ULTRA: 192 bits of security
  *
@@ -715,11 +723,14 @@ typedef enum {
 	GNUTLS_SEC_PARAM_UNKNOWN = 0,
 	GNUTLS_SEC_PARAM_LOW = 1,
 	GNUTLS_SEC_PARAM_LEGACY = 2,
-	GNUTLS_SEC_PARAM_NORMAL = 3,
+	GNUTLS_SEC_PARAM_MEDIUM = 3,
 	GNUTLS_SEC_PARAM_HIGH = 4,
-	GNUTLS_SEC_PARAM_ULTRA = 5,
+	GNUTLS_SEC_PARAM_ULTRA = 5
 } gnutls_sec_param_t;
 
+/* old name */
+#define GNUTLS_SEC_PARAM_NORMAL GNUTLS_SEC_PARAM_MEDIUM
+
 /**
  * gnutls_channel_binding_t:
  * @GNUTLS_CB_TLS_UNIQUE: "tls-unique" (RFC 5929) channel binding
@@ -799,6 +810,8 @@ gnutls_sec_param_t gnutls_pk_bits_to_sec_param(gnutls_pk_algorithm_t algo,
 const char *gnutls_sec_param_get_name(gnutls_sec_param_t param);
 unsigned int gnutls_sec_param_to_pk_bits(gnutls_pk_algorithm_t algo,
 					 gnutls_sec_param_t param);
+unsigned int
+gnutls_sec_param_to_symmetric_bits(gnutls_sec_param_t param);
 
 /* Elliptic curves */
 const char *gnutls_ecc_curve_get_name(gnutls_ecc_curve_t curve);
@@ -926,6 +939,16 @@ ssize_t gnutls_record_send_range(gnutls_session_t session,
 				 const gnutls_range_st * range);
 ssize_t gnutls_record_recv(gnutls_session_t session, void *data,
 			   size_t data_size);
+
+typedef struct mbuffer_st *gnutls_packet_t;
+
+ssize_t
+gnutls_record_recv_packet(gnutls_session_t session, 
+		   	  gnutls_packet_t *packet);
+
+void gnutls_packet_get(gnutls_packet_t packet, gnutls_datum_t *data, unsigned char *sequence);
+void gnutls_packet_deinit(gnutls_packet_t packet);
+
 #define gnutls_read gnutls_record_recv
 #define gnutls_write gnutls_record_send
 ssize_t gnutls_record_recv_seq(gnutls_session_t session, void *data,
@@ -1073,6 +1096,7 @@ int gnutls_key_generate(gnutls_datum_t * key, unsigned int key_size);
 
 /* if you just want some defaults, use the following.
  */
+
 int gnutls_priority_init(gnutls_priority_t * priority_cache,
 			 const char *priorities, const char **err_pos);
 void gnutls_priority_deinit(gnutls_priority_t priority_cache);
@@ -1231,6 +1255,8 @@ void gnutls_credentials_clear(gnutls_session_t session);
  */
 int gnutls_credentials_set(gnutls_session_t session,
 			   gnutls_credentials_type_t type, void *cred);
+int gnutls_credentials_get(gnutls_session_t session,
+			   gnutls_credentials_type_t type, void **cred);
 #define gnutls_cred_set	gnutls_credentials_set
 
 /* x.509 types */
@@ -1331,6 +1357,11 @@ int
 gnutls_certificate_set_x509_trust_file(gnutls_certificate_credentials_t
 				       cred, const char *cafile,
 				       gnutls_x509_crt_fmt_t type);
+int
+gnutls_certificate_set_x509_trust_dir(gnutls_certificate_credentials_t cred,
+				      const char *ca_dir,
+				      gnutls_x509_crt_fmt_t type);
+
 int gnutls_certificate_set_x509_trust_mem(gnutls_certificate_credentials_t
 					  res, const gnutls_datum_t * ca,
 					  gnutls_x509_crt_fmt_t type);
@@ -1453,18 +1484,10 @@ typedef int (*gnutls_is_secure_function) (const void *);
 typedef void (*gnutls_free_function) (void *);
 typedef void *(*gnutls_realloc_function) (void *, size_t);
 
-void
-gnutls_global_set_mem_functions(gnutls_alloc_function alloc_func,
-				gnutls_alloc_function secure_alloc_func,
-				gnutls_is_secure_function is_secure_func,
-				gnutls_realloc_function realloc_func,
-				gnutls_free_function free_func);
-
 void gnutls_global_set_time_function(gnutls_time_func time_func);
 
 /* For use in callbacks */
 extern gnutls_alloc_function gnutls_malloc;
-extern gnutls_alloc_function gnutls_secure_malloc;
 extern gnutls_realloc_function gnutls_realloc;
 extern gnutls_calloc_function gnutls_calloc;
 extern gnutls_free_function gnutls_free;
@@ -1583,7 +1606,7 @@ int gnutls_fingerprint(gnutls_digest_algorithm_t algo,
    * Enumeration of different random art types.
    */
 typedef enum gnutls_random_art {
-	GNUTLS_RANDOM_ART_OPENSSH = 1,
+	GNUTLS_RANDOM_ART_OPENSSH = 1
 } gnutls_random_art_t;
 
 int gnutls_random_art(gnutls_random_art_t type,
@@ -1678,6 +1701,12 @@ int gnutls_srp_base64_decode(const gnutls_datum_t * b64_data, char *result,
 int gnutls_srp_base64_decode_alloc(const gnutls_datum_t * b64_data,
 				   gnutls_datum_t * result);
 
+void
+gnutls_srp_set_server_fake_salt_seed(gnutls_srp_server_credentials_t
+				     sc,
+				     const gnutls_datum_t * seed,
+				     unsigned int salt_length);
+
 /* PSK stuff */
 typedef struct gnutls_psk_server_credentials_st
 *gnutls_psk_server_credentials_t;
@@ -1905,6 +1934,32 @@ int gnutls_certificate_verify_peers3(gnutls_session_t session,
 				     const char *hostname,
 				     unsigned int *status);
 
+/**
+ * gnutls_vdata_types_t:
+ * @GNUTLS_DT_UNKNOWN: Unknown data type.
+ * @GNUTLS_DT_DNS_HOSTNAME: The data contain a null-terminated DNS hostname.
+ * @GNUTLS_DT_KEY_PURPOSE_OID: The data contain a null-terminated key purpose OID.
+ *
+ * Enumeration of different key exchange algorithms.
+ */
+typedef enum {
+	GNUTLS_DT_UNKNOWN = 0,
+	GNUTLS_DT_DNS_HOSTNAME = 1,
+	GNUTLS_DT_KEY_PURPOSE_OID = 2
+} gnutls_vdata_types_t;
+
+typedef struct {
+	gnutls_vdata_types_t type;
+	unsigned char *data;
+	unsigned int size;
+} gnutls_typed_vdata_st;
+
+int
+gnutls_certificate_verify_peers(gnutls_session_t session,
+				gnutls_typed_vdata_st * data,
+				unsigned int elements,
+				unsigned int *status);
+
 int gnutls_certificate_verification_status_print(unsigned int status,
 						 gnutls_certificate_type_t
 						 type,
@@ -2038,7 +2093,7 @@ typedef enum {
 	GNUTLS_PIN_FINAL_TRY = (1 << 2),
 	GNUTLS_PIN_COUNT_LOW = (1 << 3),
 	GNUTLS_PIN_CONTEXT_SPECIFIC = (1 << 4),
-	GNUTLS_PIN_WRONG = (1 << 5),
+	GNUTLS_PIN_WRONG = (1 << 5)
 } gnutls_pin_flag_t;
 
 #define GNUTLS_PKCS11_PIN_USER GNUTLS_PIN_USER
@@ -2091,6 +2146,9 @@ void gnutls_certificate_set_pin_function(gnutls_certificate_credentials_t,
 					 gnutls_pin_callback_t fn,
 					 void *userdata);
 
+/* FIPS140-2 related functions */
+int gnutls_fips140_mode_enabled(void);
+
   /* Gnutls error codes. The mapping to a TLS alert is also shown in
    * comments.
    */
@@ -2280,6 +2338,12 @@ void gnutls_certificate_set_pin_function(gnutls_certificate_credentials_t,
 #define GNUTLS_E_RANDOM_DEVICE_ERROR -342
 #define GNUTLS_E_AUTH_ERROR -343
 #define GNUTLS_E_NO_APPLICATION_PROTOCOL -344
+#define GNUTLS_E_SOCKETS_INIT_ERROR -345
+
+#define GNUTLS_E_SELF_TEST_ERROR -400
+#define GNUTLS_E_NO_SELF_TEST -401
+#define GNUTLS_E_LIB_IN_ERROR_STATE -402
+#define GNUTLS_E_PK_GENERATION_ERROR -403
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
